@@ -111,12 +111,19 @@ class GnomoliciousApplet(gnome.applet.Applet):
             'url' : wT.get_widget('entry_url'),
             'desc' : wT.get_widget('entry_desc'),
             'tags' : wT.get_widget('entry_tags'),
-            'ext' : wT.get_widget('entry_ext') }
+            'ext' : wT.get_widget('entry_ext'),
+            'popular' : wT.get_widget('populartable') }
+        self.populartags_buttons = {}
+        self.populartags_nobuttons = gtk.Label(_('No popular tags retrieved for this URL'))
         self.postwin.completion = gtk.EntryCompletion()
         self.postwin.entries['tags'].set_completion(self.postwin.completion)
-        self.tags = gtk.ListStore(gobject.TYPE_STRING)
+        self.tags = gtk.ListStore(str, str)
         self.update_tags()
         self.postwin.completion.set_model(self.tags)
+        numtags = gtk.CellRendererText()
+        numtags.set_property('foreground', 'grey')
+        self.postwin.completion.pack_end(numtags)
+        self.postwin.completion.add_attribute(numtags, 'text', 1)
         self.postwin.completion.set_text_column(0)
         self.postwin.completion.set_match_func(findMatchingTag, 0)
         self.postwin.completion.set_minimum_key_length(2)
@@ -145,12 +152,66 @@ class GnomoliciousApplet(gnome.applet.Applet):
             taglist = delicious.get_tags(username, password)
             for tagdict in taglist:
                 logger.debug('Adding: %s' % tagdict['tag'])
-                self.tags.append([tagdict['tag']])
+                if int(tagdict['count']) > 1:
+                    numtag = '%s occurences' % tagdict['count']
+                else:
+                    numtag = '%s occurence' % tagdict['count']
+                self.tags.append([tagdict['tag'], numtag])
             logger.info('Updating your tags: done')
 
+    def clear_postwin(self):
+        self.postwin.entries['url'].set_text('')
+        self.postwin.entries['desc'].set_text('')
+        self.postwin.entries['tags'].set_text('')
+        buf = self.postwin.entries['ext'].get_buffer()
+        buf.delete(buf.get_start_iter(), buf.get_end_iter())
+        for tag, btn in self.populartags_buttons.items():
+            self.postwin.entries['popular'].remove(btn)
+        self.populartags_buttons = {}
+        self.postwin.entries['popular'].attach(self.populartags_nobuttons,
+                0, 15, 0, 5)
+
+    def clear_prefwin(self):
+        self.prefwin.entries['password'].set_text('')
+        self.prefwin.entries['username'].set_text('')
+
+    def get_poptags(self, url):
+        tags = {}
+        posts = delicious.get_urlposts(url)
+        for post in posts:
+            for tag in post['tags'].split():
+                if tag in tags:
+                    tags[tag] += 1
+                else:
+                    tags[tag] = 1
+        lst = tags.items()
+        lst.sort(lambda x,y: cmp(x[1], y[1]))
+        lst.reverse()
+        return lst
+
+    def add_poptags(self, tags):
+        row, col = 0, 0
+        if tags:
+            self.postwin.entries['popular'].remove(self.populartags_nobuttons)
+        for tag, num in tags:
+            btn = gtk.CheckButton(tag)
+            btn.connect('toggled', self.on_poptoggled, tag)
+            self.populartags_buttons[tag] = btn
+            self.postwin.entries['popular'].attach(btn, col, col+1,
+                    row, row+1)
+            col = (col + 1) % 3
+            if not col:
+                row += 1
+
+    ######################################################
+    #               Callbacks for signals                #
+    ######################################################
+    
     def post(self, evt, data=None):
         self.update_tags()
-        self.postwin.show()
+        self.postwin.entries['popular'].attach(self.populartags_nobuttons,
+                0, 14, 0, 5)
+        self.postwin.show_all()
 
     def preferences(self, evt, data=None):
         username = self.conf.get_string(GCONF_PATH + '/username')
@@ -166,21 +227,17 @@ class GnomoliciousApplet(gnome.applet.Applet):
                 ['Nicolas Évrard <nicoe@altern.org>'], [], '', self.logo)
         about.show()
 
-    def clear_postwin(self):
-        self.postwin.entries['url'].set_text('')
-        self.postwin.entries['desc'].set_text('')
-        self.postwin.entries['tags'].set_text('')
-        buf = self.postwin.entries['ext'].get_buffer()
-        buf.delete(buf.get_start_iter(), buf.get_end_iter())
+    def on_poptoggled(self, widget, data=None):
+        tag = data
+        toggled = widget.get_active()
+        entry_tags = self.postwin.entries['tags'].get_text().split()
+        if tag in entry_tags and not toggled:
+            entry_tags.remove(tag)
+            self.postwin.entries['tags'].set_text(' '.join(entry_tags))
+        elif tag not in entry_tags and toggled:
+            entry_tags.append(tag)
+            self.postwin.entries['tags'].set_text(' '.join(entry_tags))
 
-    def clear_prefwin(self):
-        self.prefwin.entries['password'].set_text('')
-        self.prefwin.entries['username'].set_text('')
-
-    ######################################################
-    #               Callbacks for signals                #
-    ######################################################
-    
     def on_resize_panel(self, widget, data):
         logger.debug('New size: %s' % data)
         pixbuf = self.logo
@@ -199,7 +256,10 @@ class GnomoliciousApplet(gnome.applet.Applet):
         self.postwin.entries['url'].set_text(url)
         self.postwin.entries['desc'].set_text(title)
         self.update_tags()
-        self.postwin.show()
+        logger.info('Retrieving popular tags')
+        poptags = self.get_poptags(url)
+        self.add_poptags(poptags[:15])
+        self.postwin.show_all()
 
     def on_button_press(self, widget, evt):
         if evt.type == gtk.gdk.BUTTON_PRESS and evt.button == 3:
@@ -253,7 +313,10 @@ class GnomoliciousApplet(gnome.applet.Applet):
         return gtk.TRUE
 
     def on_postwin_tags(self, widget, event):
-        logger.debug('event: %s' % event.keyval)
-        if event.keyval == 65289: # code for the tab key
-            return gtk.TRUE
+        if event.string == ' ':
+            tags = self.postwin.entries['tags'].get_text().split()
+            for tag in tags:
+                if tag in self.populartags_buttons:
+                    btn = self.populartags_buttons[tag]
+                    btn.set_active(True)
         return gtk.FALSE
